
# q-expansion of newform 8041.2.a.b, downloaded from the LMFDB on 27 April 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 8041
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[-1], [3], [-2], [-2], [-1], [4], [1], [8], [4], [-3], [-2], [4], [-6], [-1], [0], [9], [6], [-7], [4], [8], [3], [-3], [-1], [-14], [-13], [8], [2], [17], [-6], [10], [17], [-8], [12], [-5], [11], [8], [10], [-3], [-7], [-2], [-21], [1], [-1], [-10], [12], [-20], [-22], [9], [0], [15], [15], [-13], [30], [-18], [12], [26], [19], [-16], [14], [12], [-19], [0], [27], [-2], [-10], [23], [28], [4], [32], [18], [-27], [9], [28], [33], [34], [-9], [-6], [33], [25], [-25], [-19], [-20], [24], [-16], [-32], [-2], [4], [6], [22], [-16], [0], [21], [-4], [2], [17], [6], [10], [-10], [-12], [-6], [19], [30], [8], [10], [16], [10], [-20], [15], [-12], [-14], [-22], [46], [-42], [-42], [-23], [-30], [-28], [29], [-24], [25], [-49], [29], [-15], [2], [-29], [12], [11], [-8], [-17], [-23], [40], [6], [-13], [52], [-13], [-32], [-8], [4], [-14], [20], [46], [30], [-28], [3], [26], [47], [-52], [-32], [-44], [1], [-12], [-7], [36], [-42], [-16], [11], [-51], [18], [19], [-30], [38], [-43], [27], [-30], [-27], [-36], [-21], [-2], [7], [-33], [36], [32], [6], [-38], [-17], [14], [-64], [-42], [36], [1], [-9], [-12], [8], [27], [8], [0], [-41], [0], [-56], [-36], [-19], [32], [-32], [-4], [18], [-28], [52], [-19], [36], [-26], [-40], [-54], [32], [-2], [4], [44], [-47], [45], [9], [32], [-43], [9], [-48], [-12], [-26], [-11], [22], [-45], [63], [46], [5], [27], [-36], [-52], [-16], [58], [21], [51], [32], [-30], [74], [-43], [-16], [9], [58], [8], [68], [38], [36], [-8], [25], [2], [0], [-15], [33], [28], [-61], [58], [-44], [-15], [-8], [-2], [52], [-16], [63], [52], [58], [-44], [-32], [-70], [30], [60], [40], [62], [4], [-27], [-62], [-47], [10], [-12], [3], [28], [28], [-26], [4], [-64], [54], [25], [22], [70], [24], [-12], [-83], [-22], [-18], [-63], [9], [-75], [56], [-74], [18], [-42], [-16], [32], [36], [78], [-30], [30], [31], [59], [-54], [-27], [-73], [-18], [-88], [2], [82], [-56], [57], [-31], [40], [-58], [72], [59], [-64], [16], [-20], [-59], [7], [18], [-85], [54], [-58], [-38], [-66], [-63], [-36], [10], [27], [54], [-10], [-18], [0], [-12], [-55], [-72], [-50], [30], [-62], [-52], [3], [-18], [78], [-37], [-66], [-33], [-2], [-58], [-18], [-15], [69], [82], [-39], [-46], [20], [-30], [-25], [10], [6], [24], [3], [-58], [36], [-90], [36], [12], [49], [0], [-13], [5], [49], [36], [54], [-81], [2], [27], [65], [-56], [-70], [86], [-30], [57], [64], [37], [76], [24], [80], [-47], [57], [44], [-67], [-16], [23], [12], [3], [10], [7], [-34], [-43], [94], [70], [1], [-51], [46], [25], [52], [-70], [-18], [87], [-60], [12], [-27], [47], [-28], [-1], [63], [28], [46], [3], [-66], [-62], [-50], [-15], [61], [-44], [14], [-36], [-62], [70], [-12]]
